<?php
use CRM_Bij1algemeen_ExtensionUtil as E;

/**
 * Groepsverwerking Bij1
 *
 * @author Erik Hommel(CiviCooP) <erik.hommel@civicoop.org>
 * @license AGPL-3.0
 */
class CRM_Bij1algemeen_Groep {

  /**
   * Method om groep aan te maken
   * @param array $groepsData
   */
  public function maakGroep(array $groepsData) {
    if (!empty($groepsData) && isset($groepsData['name'])) {
      if (!isset($groepsData['title'])) {
        $delen = explode("_", $groepsData['name']);
        foreach ($delen as $deelKey => $deel) {
          $delen[$deelKey] = ucfirst(strtolower($deel));
        }
        $groepsData['title'] = implode(" ", $delen);
      }
      try {
        $groep = \Civi\Api4\Group::create()
          ->addValue('name', $groepsData['name'])
          ->addValue('title', $groepsData['title'])
          ->addValue('is_active', TRUE)
          ->addValue('group_type', [2])
          ->addValue('is_reserved', TRUE);
        if (isset($groepsData['description'])) {
          $groep->addValue('description', $groepsData['description']);
        }
        if (isset($groepsData['parents:name'])) {
          $groep->addValue('parents:name', [$groepsData['parents:name']]);
        }
        $groep->execute();
      }
      catch (API_Exception $ex) {
        Civi::log()->error("Kon geen groep aanmaken met data: " . json_encode($groepsData) . ", foutboodschap van API4 Group create: " . $ex->getMessage());
      }
    }
  }

  /**
   * Method om te checken of de groep bestaat
   *
   * @param string $groepsNaam
   * @return bool
   */
  public function bestaatGroep(string $groepsNaam) {
    if (!empty($groepsNaam)) {
      try {
        $groups = \Civi\Api4\Group::get()
          ->addSelect('COUNT(*) AS count')
          ->addWhere('name', '=', $groepsNaam)
          ->execute();
        foreach ($groups as $group) {
          if ($group['count'] > 0) {
            return TRUE;
          }
        }
      }
      catch (API_Exception $ex) {
        Civi::log()->error("Kon geen groep zoeken met naam " . $groepsNaam . " in " . __METHOD__ . ", foutboodschap van API4 Group get: " . $ex->getMessage());
      }
    }
    return FALSE;
  }

  /**
   * Method om leden groepen op te halen
   *
   * @return string[][]
   */
  private function haalLedenGroepen() {
    return [
      [
        'name' => "bij1_leden",
        'title' => "Alle leden",
        'description' => "Deze groep bevat alle leden van Bij1 en heeft verschillende subgroepen",
      ],
      [
        'name' => "bij1_betaling_gemist",
        'title' => "Betaling gemist",
        'description' => "Deze groep bevat alle leden die een betaling gemist hebben",
      ],
      [
        'name' => "bij1_ex_leden",
        'title' => "Ex-leden",
        'description' => "Deze groep bevat alle ex-leden",
      ],
      [
        'name' => "bij1_nieuwe_leden",
        'title' => "Nieuwe leden",
        'description' => "Deze groep bevat alle nieuwe leden",
      ],
      [
        'name' => "bij1_sociaal_tarief",
        'title' => "Sociaal tarief leden",
        'description' => "Deze groep bevat alle sociaal tarief leden",
      ],
      [
        'name' => "bij1_standaard_leden",
        'title' => "Standaard leden",
        'description' => "Deze groep bevat alle standaard leden",
      ],
    ];
  }

  /**
   * Functie om initiele ledengroepen aan te maken
   */
  public function maakLedenGroepen() {
    $ledenGroepen = $this->haalLedenGroepen();
    foreach ($ledenGroepen as $groepsData) {
      if (!$this->bestaatGroep($groepsData['name'])) {
        $this->maakGroep($groepsData);
      }
    }
    // fix probleem dat ouder groep niet ingevuld wordt
    $hoofdGroepId = $this->haalGroepsIdMetNaam('bij1_leden');
    if ($hoofdGroepId) {
      $kindGroepen = [];
      foreach ($ledenGroepen as $groepsData) {
        if ($groepsData['name'] != "bij1_leden") {
          $kindGroepId = $this->haalGroepsIdMetNaam($groepsData['name']);
          if ($kindGroepId) {
            CRM_Core_DAO::executeQuery("UPDATE civicrm_group SET parents = %1 WHERE id = %2", [
              1 => [$hoofdGroepId, "String"],
              2 => [$kindGroepId, "Integer"],
            ]);
            CRM_Core_DAO::executeQuery("INSERT INTO civicrm_group_nesting (child_group_id, parent_group_id) VALUES(%2, %1)", [
              1 => [$hoofdGroepId, "String"],
              2 => [$kindGroepId, "Integer"],
            ]);
            $kindGroepen[] = $kindGroepId;
          }
        }
      }
      CRM_Core_DAO::executeQuery("UPDATE civicrm_group SET children = %1 WHERE id = %2", [
        1 => [implode(",", $kindGroepen), "String"],
        2 => [$hoofdGroepId, "Integer"],
      ]);
    }
  }

  /**
   * Method om group_id op te halen met name
   *
   * @param string $groepsNaam
   * @return false|int
   */
  public function haalGroepsIdMetNaam(string $groepsNaam) {
    try {
      $groups = \Civi\Api4\Group::get()
        ->addSelect('id')
        ->addWhere('name', '=', $groepsNaam)
        ->execute();
      $group = $groups->first();
      if ($group['id']) {
        return (int) $group['id'];
      }
    }
    catch (API_Exception $ex) {
      Civi::log()->error(E::ts("Kon geen groepsid vinden met groepsnaam: ") . $groepsNaam . E::ts(", foutmelding van API4 Group get: ") . $ex->getMessage());
    }
    return FALSE;
  }


  /**
   * Method om contact in groep te plaatsen
   *
   * @param int $contactId
   * @param int $groepId
   * @param null $vanDatum
   * @return false
   * @throws API_Exception
   */
  public function plaatsInGroep(int $contactId, int $groepId, $vanDatum = NULL) {
    if (!empty($contactId) && !empty($groepId)) {
      if ($vanDatum) {
        if (!$vanDatum instanceof DateTime) {
          $vanDatum = new DateTime($vanDatum);
        }
      }
      if (!$this->isActiefGroepslid($contactId, $groepId)) {
        // eerst bepalen of er al een beëindigd groepslidmaatschap is
        if ($this->isVerlopenGroepslid($contactId, $groepId)) {
          \Civi\Api4\GroupContact::update()
            ->addValue('status', 'Added')
            ->addWhere('group_id', '=', $groepId)
            ->addWhere('contact_id', '=', $contactId)
            ->addWhere('status', '!=', 'Added')
            ->execute();
        }
        else {
          \Civi\Api4\GroupContact::create()
            ->addValue('status', 'Added')
            ->addValue('group_id', $groepId)
            ->addValue('contact_id',$contactId)
            ->execute();
        }
        // als er een vanDatum is subscription history bijwerken (is geen API voor!)
        // dat betekent eerst de nieuwste toevoeging bijwerken met vanDatum en daarna alle bewerkingen van een later
        // tijdstip verwijderen
        if ($vanDatum) {
          $update = "UPDATE civicrm_subscription_history SET date = %1 WHERE contact_id = %2 AND group_id = %3 AND status = %4 ORDER BY date DESC LIMIT 1";
          CRM_Core_DAO::executeQuery($update, [
            1 => [$vanDatum->format("Y-m-d"), "String"],
            2 => [$contactId, "Integer"],
            3 => [$groepId, "Integer"],
            4 => ["Added", "String"],
          ]);
          $delete = "DELETE FROM civicrm_subscription_history WHERE date > %1 AND contact_id = %2 AND group_id = %3";
          CRM_Core_DAO::executeQuery($delete, [
            1 => [$vanDatum->format("Y-m-d"), "String"],
            2 => [$contactId, "Integer"],
            3 => [$groepId, "Integer"],
          ]);
        }
      }
    }
    return FALSE;
  }

  /**
   * Method om te bekijken of een contact in een groep zit
   *
   * @param int $contactId
   * @param int $groepId
   * @return bool
   */
  public function isActiefGroepslid(int $contactId, int $groepId) {
    try {
      $groupContacts = \Civi\Api4\GroupContact::get()
        ->addSelect('COUNT(*) AS count')
        ->addWhere('group_id', '=', $groepId)
        ->addWhere('contact_id', '=', $contactId)
        ->addWhere('status', '=', 'Added')
        ->execute();
      $groupContact = $groupContacts->first();
      if (isset($groupContact['count']) && $groupContact['count'] > 0) {
        return TRUE;
      }
    }
    catch (API_Exception $ex) {
    }
    return FALSE;
  }

  /**
   * Method om te bekijken of een contact in een groep heeft gezeten
   * (alleen te gebruiken in combinatie met functie isActiefGroepslid - dus nadat isActiefGroepslid al FALSE heeft opgeleverd)
   *
   * @param int $contactId
   * @param int $groepId
   * @return bool
   */
  public function isVerlopenGroepslid(int $contactId, int $groepId) {
    try {
      $groupContacts = \Civi\Api4\GroupContact::get()
        ->addSelect('COUNT(*) AS count')
        ->addWhere('group_id', '=', $groepId)
        ->addWhere('contact_id', '=', $contactId)
        ->addWhere('status', '!=', 'Added')
        ->execute();
      $groupContact = $groupContacts->first();
      if (isset($groupContact['count']) && $groupContact['count'] > 0) {
        return TRUE;
      }
    }
    catch (API_Exception $ex) {
    }
    return FALSE;
  }

  /**
   * Method om contact uit groep te verwijderen (remove or delete)
   *
   * @param int $contactId
   * @param int $groepId
   * @param bool $delete
   */
  public function verwijderUitGroep(int $contactId, int $groepId, bool $delete = FALSE) {
    if (!empty($contactId) && !empty($groepId)) {
      $datum = new DateTime('now');
      if ($delete) {
        $status = "Deleted";
        $delete = "DELETE FROM civicrm_group_contact WHERE contact_id = %1 AND group_id = %2";
        CRM_Core_DAO::executeQuery($delete, [
          1 => [$contactId, "Integer"],
          2 => [$groepId, "Integer"],
        ]);
      }
      else {
        $status = "Removed";
        $gc = "UPDATE civicrm_group_contact SET status = %1 WHERE contact_id = %2 AND group_id = %3";
        CRM_Core_DAO::executeQuery($gc, [
          1 => [$status, "String"],
          2 => [$contactId, "Integer"],
          3 => [$groepId, "Integer"],
        ]);
      }
      $insert = "INSERT INTO civicrm_subscription_history (contact_id, group_id, status, date) VALUES(%1, %2, %3, %4)";
      CRM_Core_DAO::executeQuery($insert, [
        1 => [$contactId, "Integer"],
        2 => [$groepId, "Integer"],
        3 => [$status, "String"],
        4 => [$datum->format('Y-m-d H:i:s'), "String"],
      ]);

    }
  }

}
